//
// Created: 1/31/03, Klaus Salchner
// created this abstraction class which abstracts all data access related code; implemented its
// RetrieveSecurityInformation, CheckuserNameAndPassword and the basic data provider logic
//
// Updated: 2/1/03, Klaus Salchner
// Added RetrieveUserInformation
//
using System;
using System.Data;
using System.Data.SqlClient;
using System.Data.Odbc;
using System.Data.OleDb;
using System.Data.OracleClient;
using System.Configuration;
using System.Globalization;
using System.Resources;
using System.Reflection;
using System.Collections;

namespace RetrieveSecurity
{
	/// <summary>
	/// define our custom exception we are throwing whenever we miss config data or run into any 
	/// other issues in the DataLayer class
	/// </summary>
	[Serializable]
	public class DataLayerException : ApplicationException
	{
		/// <summary>
		/// we need to provide our own constructor but we just pass on the message to the base constructor 
		/// </summary>
		/// <param name="ErrorMessage"></param>
		public DataLayerException(string ErrorMessage) : base (ErrorMessage)
		{
		}
	}

	/// <summary>
	/// the data layer class abstracts all data-base and resource file interactions from the
	/// rest of the code
	/// </summary>
	public sealed class DataLayer
	{
		private static string DbConnectionString = "ConnectionString";
		private static string DatabaseTypeString = "DatabaseType";
		private static string SqlCheckUserNameAndPassword = "SqlCheckUserNameAndPassword";
		private static string MissingConnectionString = "MissingConnectionString";
		private static string MissingSqlString = "MissingSqlString";
		private static string MissingParamString = "MissingParamString";
		private static string RecourceBaseName = "RetrieveSecurity.Resources";
		private static string DataAccessException = "DataAccessException";
		private static string UserNameParamName = "UserNameParamName";
		private static string PasswordParamName = "PasswordParamName";
		private static string ListOfSecurityGroups = "ListOfSecurityGroups";
		private static string ListOfSecurityRights = "ListOfSecurityRights";
		private static string UserIdParamName = "UserIdParamName";
		private static string ErrorReadingSecurityGroups = "ErrorReadingSecurityGroups";
		private static string ErrorReadingSecurityRights = "ErrorReadingSecurityRights";
		private static string UserInformation = "UserInformation";
		private static string ErrorReadingUserInfo = "ErrorReadingUserInfo";
		private static string UserNameKey = "UserName";
		private static string FirstNameKey = "FirstName";
		private static string LastNameKey = "LastName";

		/// <summary>
		/// define the possible database types we support; drives which data provider will be
		/// used
		/// </summary>
		private enum DatabaseType
		{
			Sql = 1,
			Oracle,
			OleDb,
			Odbc
		}

		/// <summary>
		/// makes the constructor private so we can't instantiate an object of this
		/// class; because all the memebers are static
		/// </summary>
		private DataLayer()
		{
		}

		/// <summary>
		/// get the request resource string from the resource file
		/// </summary>
		/// <param name="StringResourceName">the string name</param>
		/// <returns>the string requested</returns>
		public static string GetString(string ResourceName)
		{
			// get the resource manager which has access to the resource table
			ResourceManager Resources = new ResourceManager(RecourceBaseName,Assembly.GetExecutingAssembly());

			// retrieve the requested string and return it
			return Resources.GetString(ResourceName,CultureInfo.CurrentUICulture);
		}

		/// <summary>
		/// reads the connection string from the .config file and then returns a valid 
		/// connection object; but we program against the DB interfaces as we allow to
		/// configure what kind of data provider should be used - Sql, Oracle, OleDb or Odbc
		/// </summary>
		/// <returns>retuns a valid connection object</returns>
		private static IDbConnection GetConnectionOject()
		{
			string ConnectionString = ConfigurationSettings.AppSettings[DbConnectionString];
			string DbTypeString = ConfigurationSettings.AppSettings[DatabaseTypeString];

			// check that we have a connection string otherwise throw an exception
			if ((ConnectionString == null) || (ConnectionString.Length <= 0))
				throw new DataLayerException(GetString(MissingConnectionString));

			// check that we have a database type set otherwise set it to the default which is Sql
			if ((DbTypeString == null) || (DbTypeString.Length <= 0))
				DbTypeString = DatabaseType.Sql.ToString();

			// get the database type and based on that create the connection object
			switch ((DatabaseType)Convert.ToByte(DbTypeString,CultureInfo.InvariantCulture))
			{
				// we use the Sql data provider
				case DatabaseType.Sql:
					return (IDbConnection)new SqlConnection(ConnectionString);

				// we use the Oracle data provider
				case DatabaseType.Oracle:
					return (IDbConnection)new OracleConnection(ConnectionString);

				// we use the OleDb data provider
				case DatabaseType.OleDb:
					return (IDbConnection)new OleDbConnection(ConnectionString);

				// we use the Odbc data provider
				case DatabaseType.Odbc:
					return (IDbConnection)new OdbcConnection(ConnectionString);

				// if not specified or not recognized value use the Sql data provider by default
				default:
					return (IDbConnection)new SqlConnection(ConnectionString);
			}
		}

		/// <summary>
		/// create a command object and set the SQL statement we want to execute
		/// </summary>
		/// <param name="DbConnection">gets the database connection to use</param>
		/// <param name="ConfigSettingName">the name of the config setting to read the SQL statement from</param>
		/// <returns>returns a valid command object</returns>
		private static IDbCommand CreateCommandObject(IDbConnection DbConnection, string ConfigSettingName)
		{
			// create a command object which we can use to check for the exitence of the username & pwd
			IDbCommand DbCommand = DbConnection.CreateCommand();
			DbCommand.CommandText = ConfigurationSettings.AppSettings[ConfigSettingName];
			DbCommand.CommandType = CommandType.Text;

			// check that we have a valid Sql statement to execute
			if ((DbCommand.CommandText == null) || (DbCommand.CommandText.Length <= 0))
				throw new DataLayerException(GetString(MissingSqlString));

			// returns the command object
			return DbCommand;
		}

		/// <summary>
		/// create a new data parameter and add it to the command object
		/// </summary>
		/// <param name="DbCommand">the command object we already created before</param>
		/// <param name="ConfigSettingName">the name of the config setting to read the param name from</param>
		/// <returns></returns>
		private static IDataParameter AddParameter(IDbCommand DbCommand,string ConfigSettingName,string Value)
		{
			// add the username to the Sql 
			IDbDataParameter DbParam = DbCommand.CreateParameter();
			DbParam.ParameterName = ConfigurationSettings.AppSettings[ConfigSettingName];
			DbParam.Value = Value;
			DbCommand.Parameters.Add(DbParam);

			// check that we have a valid parameter name to execute
			if ((DbParam.ParameterName == null) || (DbParam.ParameterName.Length <= 0))
				throw new DataLayerException(GetString(MissingParamString));

			// returns the Db param we created so it can be further manipulated
			return DbParam;
		}

		/// <summary>
		/// checks whether this combination of username & pwd exists in the User table; we are
		/// case sensitive; 
		/// </summary>
		/// <param name="UserName">the user name</param>
		/// <param name="Password">the password</param>
		/// <returns>returns true if valid username & pwd</returns>
		public static bool CheckUserNameAndPassword(string UserName, string Password, out int UserId)
		{
			Object ReturnValue;

			// get a valid connection object
			IDbConnection DbConnection = GetConnectionOject();

			// create a command object which we can use to check for the exitence of the username & pwd
			IDbCommand DbCommand = CreateCommandObject(DbConnection,SqlCheckUserNameAndPassword);

			// add the username to the Sql 
			AddParameter(DbCommand,UserNameParamName,UserName);

			// add the password to the Sql 
			AddParameter(DbCommand,PasswordParamName,Password);

			try
			{
				// open the datase, query for the username and password and then close the connection
				DbConnection.Open();
				ReturnValue = DbCommand.ExecuteScalar();
			}

			// we had issue retrieving the user name and password so we throw a custom exception
			// to tell the caller that we failed to check
			catch (Exception e)
			{
				throw new DataLayerException(GetString(DataAccessException));
			}

			// make sure that the database connection gets closed
			finally
			{
				DbConnection.Close();
			}

			// if the return value from ExecuteScalar is null then we did not find the username & password
			if (ReturnValue == null)
			{
				UserId = 0;
				return false;
			}

			// otherwise we found it and return the ID of the user
			else
			{
				UserId = Convert.ToInt16(ReturnValue,CultureInfo.InvariantCulture);
				return true;
			}
		}

		/// <summary>
		/// delegate which can be used to pass along to ReadValuesIntoHashtable; if a delegate is provided
		/// then it is called otherwise ReadValuesIntoHashtable implements a default behavior how to add
		/// information to the NameValues hash-table
		/// </summary>
		private delegate void AddRecordInfo(Hashtable NameValues,IDataReader DataReader);

		/// <summary>
		/// read the data by executing the command object we pass on and then read all the data into a
		/// hash-table we return; first column is always the ID and second always the Name; the ID is
		/// used as the key and the Name as the value in the hash-table
		/// </summary>
		/// <param name="DbConnection">the database connection to use</param>
		/// <param name="DbCommand">the command object to use</param>
		/// <param name="ExceptionStringResourceName">the user-friendly exception; the resource string name we read</param>
		/// <returns>the hash-table with the ID/Name values; returned to the caller</returns>
		private static Hashtable ReadValuesIntoHashtable(IDbConnection DbConnection, IDbCommand DbCommand,string ExceptionStringResourceName,AddRecordInfo AddRecordInfoDelegate)
		{
			// create the empty hashtable
			Hashtable NameValues = new Hashtable();

			try
			{
				// open the database and execute the command object to get the list of data we are
				// querying for
				DbConnection.Open();
				IDataReader DataReader = DbCommand.ExecuteReader();

				// now read all the information belonging to the user into a hash-table; the first column
				// contains always the ID and the second column the name; and we read it one-to-one into
				// the hashtable
				while (DataReader.Read())
				{
					// if we a delegate has been passed along then call it to add the values to the 
					// hash-table
					if (!(AddRecordInfoDelegate == null))
						AddRecordInfoDelegate(NameValues,DataReader);
					
					// otherwise implement a standard way how values are added to the hash-table
					else
						NameValues.Add(DataReader.GetInt32(0),DataReader.GetString(1));	
				}
			}

			// if we run into any error reading the into, then re-throw a user-friendly exception
			catch (Exception e)
			{
				throw new DataLayerException(GetString(ExceptionStringResourceName));
			}

			// close the database connection
			finally
			{
				DbConnection.Close();
			}

			// the hash-table with the info
			return NameValues;
		}

		/// <summary>
		/// reads all the security information for this user from the database; we separately return the
		/// unique list of groups the user is belonging to plus a unique list of security rights the 
		/// user has; this information is used for the IPrincipal object
		/// </summary>
		/// <param name="UserId">the user ID</param>
		/// <param name="SecurityGroups">the list of security groups</param>
		/// <param name="SecurityRights">the list of security rights</param>
		public static void RetrieveSecurityInformation(int UserId, out Hashtable SecurityGroups, out Hashtable SecurityRights)
		{
			// get a valid connection object
			IDbConnection DbConnection = GetConnectionOject();

			// gets the command object to read the list of secuirty groups the user belongs too
			IDbCommand DbCommand = CreateCommandObject(DbConnection,ListOfSecurityGroups);

			// add the user id to the command object
			AddParameter(DbCommand,UserIdParamName,Convert.ToString(UserId,CultureInfo.InvariantCulture));

			// reads all the security group info into a hash-table 
			SecurityGroups = ReadValuesIntoHashtable(DbConnection,DbCommand,ErrorReadingSecurityGroups,null);

			// gets the command object to read the list of secuirty rights the user has
			DbCommand = CreateCommandObject(DbConnection,ListOfSecurityRights);

			// add the user id to the command object
			AddParameter(DbCommand,UserIdParamName,Convert.ToString(UserId,CultureInfo.InvariantCulture));
			
			// reads all the all the security rights info into a hash-table 
			SecurityRights = ReadValuesIntoHashtable(DbConnection,DbCommand,ErrorReadingSecurityRights,null);
		}

		/// <summary>
		/// read the information associated with the user; this information is then used for the 
		/// IIdentity object
		/// </summary>
		/// <param name="UserId">the user id</param>
		/// <param name="UserInfo">the list of user information</param>
		public static void RetrieveUserInformation(int UserId, out Hashtable UserInfo)
		{
			// get a valid connection object
			IDbConnection DbConnection = GetConnectionOject();

			// gets a command obejct used to read all the user information associated with the user
			IDbCommand DbCommand = CreateCommandObject(DbConnection,UserInformation);

			// add the user id to the command object
			AddParameter(DbCommand,UserIdParamName,Convert.ToString(UserId,CultureInfo.InvariantCulture));

			// read all the user info into the has-table
			UserInfo = ReadValuesIntoHashtable(DbConnection,DbCommand,ErrorReadingUserInfo,new AddRecordInfo(AddUserInfoToHashtable));

			// the IIdenty object always expects a UserName entry in the hash-table so we always
			// create it here if it doesn't exist already
			if (UserInfo[UserNameKey] == null)
				UserInfo[UserNameKey] = ((UserInfo[FirstNameKey] == null) ? String.Empty : UserInfo[FirstNameKey]) + " " + 
										((UserInfo[LastNameKey]  == null) ? String.Empty : UserInfo[LastNameKey] ) ;
		}

		/// <summary>
		/// a delagate used by RetrieveUserInformation so we can implement our custom way how we
		/// add user information to the user info hash-table
		/// </summary>
		/// <param name="NameValues"></param>
		/// <param name="DataReader"></param>
		private static void AddUserInfoToHashtable(Hashtable NameValues,IDataReader DataReader)
		{
			for (int Count=0; Count<DataReader.FieldCount; Count++)
				NameValues.Add(DataReader.GetName(Count),DataReader.GetValue(Count));
		}
	}
}
